/* $Header: /cvsroot/dhcp-agent/dhcp-agent/src/dhcp-cache-entry.c,v 1.7 2003/03/25 03:01:04 actmodern Exp $
 * 
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * Cache entry object:
 * 
 * We use this object to hold a name/value duplet.
 * 
 */

#define MODULE_NAME "dhcp-cache-entry"

#include "dhcp-local.h"
#include "dhcp-libutil.h"

#include "dhcp-cache-entry.h"

/* * * * * * * * * * * * * * *
 * constructors/destructors  *
 * * * * * * * * * * * * * * */

cache_entry_t *create_cache_entry(uint8_t tag, const char *name, const char *value)
{
    cache_entry_t *cache;

    cache = xmalloc(sizeof(cache_entry_t));
    cache->name = xstrdup(name);
    cache->value = xstrdup(value);
    cache->tag = tag;

    return cache;
}

void cache_entry_destroy(cache_entry_t *cache)
{
    xfree(cache->name);
    xfree(cache->value);
    xfree(cache);

    return;
}

/* list purge compatible option. */
void cache_entry_destroy_l(void *data)
{
    cache_entry_destroy(data);
}

/* purge list of cache entries. */
void cache_entry_purge_list(list_t *cache_list)
{
    list_destroy(cache_list, cache_entry_destroy_l);
    return;
}
