/* $Header: /cvsroot/dhcp-agent/dhcp-agent/src/dhcp-conf-var.c,v 1.1 2003/03/25 03:05:52 actmodern Exp $
 * 
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#define MODULE_NAME "dhcp-conf-var"

#include "dhcp-local.h"
#include "dhcp-libutil.h"

#include "dhcp-conf-var.h"

/* * * * * * * * * * * * * * * * * * * * * * * * *
 * conf_var_t is a super light variable holder.  *
 * We just want to hold a name and a pointer to  *
 * data.                                         *
 *                                               *
 * * * * * * * * * * * * * * * * * * * * * * * * */

/* create/destroy */

conf_var_t *dhcp_conf_var_create(var_type_t var_type, void *val)
{
    conf_var_t *var;

    var = xmalloc(sizeof(conf_var_t));
    var->type = var_type;
    var->val = val;

    return var;
}

void dhcp_conf_var_destroy(conf_var_t *var)
{
    xfree(var->val);
    xfree(var);
}

/* utility destroy. we call this when we don't want to free the
 * value because we've called something like
 * dhcp_conf_var_overwrite() with it. */

void dhcp_conf_var_destroy_no_val_free(conf_var_t *var)
{
    xfree(var);
}

void dhcp_conf_var_destroy_l(void *data)
{
    conf_var_t *var = data;

    dhcp_conf_var_destroy(var);
    return;
}

/* accessors. */

void *dhcp_conf_var_get_val(conf_var_t *var)
{
    return var->val;
}

var_type_t dhcp_conf_var_get_type(conf_var_t *var)
{
    return var->type;
}

/* free up and overwrite old value. */
void dhcp_conf_var_overwrite(conf_var_t *var, var_type_t var_type,
                             void *val)
{
    xfree(var->val);

    var->type = var_type;
    var->val = val;

    return;
}
