/* $Header: /cvsroot/dhcp-agent/dhcp-agent/src/dhcp-icmp.c,v 1.3 2002/11/16 00:23:43 actmodern Exp $
 * 
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#define MODULE_NAME "dhcp-icmp"

#include "dhcp-local.h"
#include "dhcp-libutil.h"
#include "dhcp-librawnet.h"

#include "dhcp-align.h"

icmp_obj *icmp_create(void)
{
    icmp_obj *icmp;

    icmp = xmalloc(sizeof(icmp_obj));

    return icmp;
}

void icmp_destroy(icmp_obj * icmp)
{
    xfree(icmp);
}

/* get routines. */
uint8_t icmp_get_type(icmp_obj * icmp)
{
    return (icmp->icmp_header.icmp_type);
}

uint32_t icmp_mask_get_mask(icmp_obj * icmp)
{
    return (icmp->icmp_msg.mask.icmp_mask);
}

/* set routines. */
void icmp_set_type(icmp_obj * icmp, uint8_t type)
{
    icmp->icmp_header.icmp_type = type;
}

void icmp_set_code(icmp_obj * icmp, uint8_t code)
{
    icmp->icmp_header.icmp_code = code;
}

void icmp_clear_cksum(icmp_obj * icmp)
{
    memset(&icmp->icmp_header.icmp_cksum, 0, sizeof(icmp->icmp_header.icmp_cksum));
}

void icmp_mask_set_id(icmp_obj * icmp, uint32_t id)
{
    icmp->icmp_msg.mask.icmp_id = htonl(id);
}

void icmp_mask_set_seq(icmp_obj * icmp, uint32_t seq)
{
    icmp->icmp_msg.mask.icmp_seq = htonl(seq);
}

void icmp_mask_set_mask(icmp_obj * icmp, uint32_t mask)
{
    icmp->icmp_msg.mask.icmp_mask = mask;
}

void icmp_echo_set_id(icmp_obj * icmp, uint16_t id)
{
    icmp->icmp_msg.echo.icmp_id = htons(id);
}

void icmp_echo_set_seq(icmp_obj * icmp, uint16_t seq)
{
    icmp->icmp_msg.echo.icmp_seq = htons(seq);
}

static void icmp_write_mask_packet_image(icmp_obj * icmp, unsigned char *packet)
{
    memcpy(packet, &icmp->icmp_msg.mask.icmp_id, sizeof(icmp->icmp_msg.mask.icmp_id));
    packet += sizeof(icmp->icmp_msg.mask.icmp_id);

    memcpy(packet, &icmp->icmp_msg.mask.icmp_seq, sizeof(icmp->icmp_msg.mask.icmp_seq));
    packet += sizeof(icmp->icmp_msg.mask.icmp_seq);

    memcpy(packet, &icmp->icmp_msg.mask.icmp_mask, sizeof(icmp->icmp_msg.mask.icmp_mask));

    return;
}

static void icmp_write_echo_packet_image(icmp_obj * icmp, unsigned char *packet)
{
    memcpy(packet, &icmp->icmp_msg.echo.icmp_id, sizeof(icmp->icmp_msg.echo.icmp_id));
    packet += sizeof(icmp->icmp_msg.echo.icmp_id);

    memcpy(packet, &icmp->icmp_msg.echo.icmp_seq, sizeof(icmp->icmp_msg.echo.icmp_seq));

    return;
}

void icmp_write_packet_image(icmp_obj * icmp, unsigned char *packet)
{

    memcpy(packet, &icmp->icmp_header.icmp_type, sizeof(icmp->icmp_header.icmp_type));
    packet += sizeof(icmp->icmp_header.icmp_type);

    memcpy(packet, &icmp->icmp_header.icmp_code, sizeof(icmp->icmp_header.icmp_code));
    packet += sizeof(icmp->icmp_header.icmp_code);

    memcpy(packet, &icmp->icmp_header.icmp_cksum, sizeof(icmp->icmp_header.icmp_cksum));
    packet += sizeof(icmp->icmp_header.icmp_cksum);

    switch (icmp->icmp_header.icmp_type) {


    case ICMP_ECHO: /* fall through */
    case ICMP_ECHOREPLY:

        icmp_write_echo_packet_image(icmp, packet);
        break;

    case ICMP_MASK: /* fall through. */
    case ICMP_MASKREPLY:

        icmp_write_mask_packet_image(icmp, packet);
        break;

    default:
        /* major error if we actually get here. */
        FATAL_MESSAGE
            ("icmp_write_packet_image() called with improper settings on icmp packet! aborting!");
    }

    return;
}

int icmp_read_packet_image(icmp_obj * icmp, const unsigned char *packet, int len)
{
    if(len < ICMP_LEN_MIN)
        return -1;

    align_icmp(packet, &icmp->icmp_header);
    packet += ICMP_HDR_LEN;
    len -= ICMP_HDR_LEN;

    switch (icmp->icmp_header.icmp_type) {

    case ICMP_ECHO:
    case ICMP_ECHOREPLY:

        align_icmp_echo(packet, &icmp->icmp_msg.echo);
        return 0;

    case ICMP_MASK:
    case ICMP_MASKREPLY:

        align_icmp_mask(packet, &icmp->icmp_msg.mask);
        return 0;

    default:
        /* can't handle any other type. */
        break;
    }

    return -1;
}
