/* $Header: /cvsroot/dhcp-agent/dhcp-agent/src/dhcp-lease.c,v 1.2 2003/07/06 05:37:44 actmodern Exp $
 *
 * Copyright 2002 Thamer Alharbash
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#define MODULE_NAME "dhcp-lease"

#include "dhcp-local.h"
#include "dhcp-libutil.h"
#include "dhcp-librawnet.h"

#include "dhcp-lease.h"
#include "dhcp-option.h"

/* * * * * * * * * * *
 * public interface. *
 * * * * * * * * * * */

lease_constraint_t *lease_constraint_create(int constraint_type, void *data)
{
    lease_constraint_t *constraint = xcalloc(sizeof(lease_constraint_t));
    char *hostname;
    list_t *address_range;
    //    ip_addr_t range_top, range_bottom;

    constraint->constraint_type = constraint_type;

    switch(constraint_type) {

    case LEASE_CONSTRAINT_HARDWARE_ADDRESS:
        memcpy(&constraint->data.hw_address.data, data, ETH_ADDR_LEN);
        break;

    case LEASE_CONSTRAINT_HOSTNAME:
        hostname = data;
        constraint->data.hostname = xstrdup(hostname);
        break;

    case LEASE_CONSTRAINT_ADDRESS_RANGE:
        address_range = data;

        hostname = data;
        constraint->data.hostname = xstrdup(hostname);
        break;

    default:
        FATAL_MESSAGE("illegal constraint type passed. this is a bug report me.");
    }

    return constraint;
}

void lease_constraint_destroy(lease_constraint_t *constraint)
{
    switch(constraint->constraint_type) {

    case LEASE_CONSTRAINT_HOSTNAME:
        xfree(constraint->data.hostname);
        break;

    case LEASE_CONSTRAINT_ADDRESS_RANGE:               /* fall through. */
    case LEASE_CONSTRAINT_HARDWARE_ADDRESS:            /* fall through. */
    case LEASE_CONSTRAINT_NONE:                        /* fall through. */
    default:
        break;

    }

    xfree(constraint);
    return;
}

lease_definition_t *lease_definition_create(lease_constraint_t *constraint,
                                            ip_addr_t ip_addr_range,
                                            list_t *options,
                                            uint32_t lease_expiry,
                                            uint32_t renew_time,
                                            uint32_t rebind_time)
{
    dhcp_opt_t *opt, *opt_copy;
    lease_definition_t *lease_def;

    lease_def = xcalloc(sizeof(lease_definition_t));

    lease_def->constraint = constraint;
    //    lease_def->addr_range = ip_addr_range;

    /* copy our dhcp options we'll pass. */

    lease_def->options = list_create();

    list_rewind(options);
    while((opt = list_next(options)) != NULL) {

        opt_copy = dhcp_option_copy(opt);

        list_add_to_end(lease_def->options, opt_copy);
    }

    /* FIXME: set renew/rebind to defaults if they are passed as zero. */

    lease_def->lease_expiry = lease_expiry;
    lease_def->renew_time = lease_expiry;
    lease_def->rebind_time = lease_expiry;

    return lease_def;
}

void lease_definition_destroy(lease_definition_t *lease_def)
{
    if(lease_def->constraint != NULL)
        lease_constraint_destroy(lease_def->constraint);

    dhcp_opt_destroy_option_list(lease_def->options);

    xfree(lease_def);

    return;
}
