/* $Header: /cvsroot/dhcp-agent/dhcp-agent/src/dhcp-option-convert.c,v 1.6 2003/06/27 03:16:27 actmodern Exp $
 * 
 * Copyright 2002, 2003 Thamer Alharbash <tmh@whitefang.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

/* FIXME: still a tad messy and a bit over redundant. but it
 * works for now until release time.  document and prune anything
 * that needs pruning. */

#define MODULE_NAME "dhcp-option-convert"

#include "dhcp-local.h"
#include "dhcp-libutil.h"
#include "dhcp-librawnet.h"

#include "dhcp-option.h"
#include "dhcp-option-convert.h"
#include "dhcp-limits.h"

/* *  *  *  *  *  *  * * 
 * utility routines.   *
 * * * * * * * * * * * */

/* count internal string atoms, and mark with sentinals. this
 * wrecks the string so it should only be used on a copy.
 * FIXME: this works but really should be a stringbuffer thing. */
static size_t count_internal_string_atoms_and_mark(char *input)
{
    char *ptr;
    size_t num = 0;

    ptr = input;
    while(*ptr)  {

        switch(*ptr)  {

        case ';':
            num++;
            *ptr = 0;
            ptr++;

        case '\\':

            if((*ptr + 1) == ';')  {
                ptr += 2; /* skip. */
            } 

        default: /* fall through. */
            ptr++;
            break;
        }
    }

    if(num == 0 && *input != 0)
        num++; /* we have at least one. */

    return num;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * From internal to network format:                              *
 *                                                               *
 *                                                               *
 * uint8, int8 are just copied verbatim since they're octets     *
 * which need no marshalling.                                    *
 *                                                               *
 * uint16, int16 are htons as they are copied over.              *
 *                                                               *
 * uint32, int32 are htonl as they are copied over.              *
 *                                                               *
 * signed are wrapped around unsigned conversions since          *
 * marshalling should not interfere with signedness.             *
 *                                                               *
 * copying from a list is a bit more ugly. here we redo          *
 * everything only we're copying from a list to contigenous      *
 * memory.                                                       *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* make a copy of an unsigned byte. */
static uint8_t *uint8_copy_to_network(const void *data, size_t num)
{
    uint8_t *uint8_copy;

    uint8_copy = xmalloc((sizeof(uint8_t) * num));
    memcpy(uint8_copy, data, (sizeof(uint8_t) * num));

    return uint8_copy;
}

/* make a copy of an signed byte. */
static int8_t *int8_copy_to_network(const void *data, size_t num)
{
    return (int8_t *)uint8_copy_to_network(data, num); /* just cast. since it doesn't matter. */
}

/* make a copy of an unsigned 16-bit integer to network. */
static uint16_t *uint16_copy_to_network(const void *data, size_t num)
{
    const uint16_t *uint16_data;
    uint16_t *uint16_copy;
    size_t i;

    uint16_data = data;
    uint16_copy = xmalloc((sizeof(uint16_t) * num));

    for(i = 0;i < num;i ++) {
        uint16_copy[i] = htons(uint16_data[i]);
    }

    return uint16_copy;
}

/* make a copy of an signed 16-bit integer. */
static int16_t *int16_copy_to_network(const void *data, size_t num)
{
    /* just cast. since it doesn't matter as long as we're just copying. */
    return (int16_t *)uint16_copy_to_network(data, num);
}

/* make a copy of an unsigned 32-bit integer. */
static uint32_t *uint32_copy_to_network(const void *data, size_t num)
{
    const uint32_t *uint32_data;
    uint32_t *uint32_copy;
    size_t i;

    uint32_data = data;
    uint32_copy = xmalloc((sizeof(uint32_t) * num));

    for(i = 0;i < num;i ++) {
        uint32_copy[i] = htonl(uint32_data[i]);
    }

    return uint32_copy;
}

/* make a copy of an signed 16-bit integer. */
static int32_t *int32_copy_to_network(const void *data, size_t num)
{
    /* just cast. since it doesn't matter as long as we're just copying. */
    return (int32_t *)uint32_copy_to_network(data, num); 
}

static void *int_list_copy_to_network(list_t *list, size_t size, uint8_t sign)
{
    void *datum;
    void *ret_data;
    size_t num;

    /* get rid of compiler warnings. these will be initialized but the compiler may not see it .*/

    uint8_t  *uint8_data = NULL;
    uint16_t *uint16_data = NULL;
    uint32_t *uint32_data = NULL;

    int8_t  *int8_data = NULL;
    int16_t *int16_data = NULL;
    int32_t *int32_data = NULL;

    uint8_t  *uint8_ptr = NULL;
    uint16_t *uint16_ptr = NULL;
    uint32_t *uint32_ptr = NULL;

    int8_t  *int8_ptr = NULL;
    int16_t *int16_ptr = NULL;
    int32_t *int32_ptr = NULL;

    num = list_get_len(list);

    /* create datum. it's size * num, but we need to assign it to the right pointer. */
    if(sign)  {

        switch(size)  {

        case sizeof(int8_t):
            int8_data = xmalloc(sizeof(int8_data) * num);
            int8_ptr = int8_data;
            ret_data = int8_data;
            break;

        case sizeof(int16_t):
            int16_data = xmalloc(sizeof(int16_data) * num);
            int16_ptr = int16_data;
            ret_data = int16_data;
            break;

        case sizeof(int32_t):
            int32_data = xmalloc(sizeof(int32_data) * num);
            int32_ptr = int32_data;
            ret_data = int32_data;
            break;

        default:
            FATAL_MESSAGE("illegal size passed for conversion. this is a bug report me.");
            exit(1);
        }

    } else {

        switch(size)  {

        case sizeof(uint8_t):
            uint8_data = xmalloc(sizeof(uint8_t) * num);
            uint8_ptr = uint8_data;
            ret_data = uint8_data;
            break;

        case sizeof(uint16_t):
            uint16_data = xmalloc(sizeof(uint16_t) * num);
            uint16_ptr = uint16_data;
            ret_data = uint16_data;
            break;

        case sizeof(uint32_t):
            uint32_data = xmalloc(sizeof(uint32_t) * num);
            uint32_ptr = uint32_data;
            ret_data = uint32_data;
            break;

        default:
            FATAL_MESSAGE("illegal size passed for conversion. this is a bug report me.");
            exit(1);
        }
    }

    while((datum = list_next(list)) != NULL)  {

        if(sign)  {

            switch(size)  {

            case sizeof(int8_t):
                *int8_ptr = *(int8_t *)datum;
                int8_ptr++;
                break;

            case sizeof(int16_t):
                *int16_ptr = htons(*(int16_t *)datum);
                int16_ptr++;
                break;

            case sizeof(int32_t):
                memcpy(int32_ptr, datum, sizeof(int32_t));
                *int32_ptr = htons(*(int32_t *)datum);
                int32_ptr++;
                break;

            default:
                FATAL_MESSAGE("illegal size passed for conversion. this is a bug report me.");
                exit(1);
            }

        }  else  {

            switch(size)  {

            case sizeof(uint8_t):
                memcpy(uint8_ptr, datum, sizeof(uint8_t));
                uint8_ptr++;
                break;

            case sizeof(uint16_t):
                *uint16_ptr = htons(*(uint16_t *)datum);
                uint16_ptr++;
                break;

            case sizeof(uint32_t):
                *uint32_ptr = htons(*(uint32_t *)datum);
                uint32_ptr++;
                break;

            default:
                FATAL_MESSAGE("illegal size passed for conversion. this is a bug report me.");
                exit(1);
            }
        }
    }

    return ret_data;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * From network to internal format:                              *
 *                                                               *
 * uint8, int8 are just copied verbatim since they're octets     *
 * which need no marshalling. we can call the uint8              *
 * copy to network routines since they do the same thing.        *
 *                                                               *
 * uint16, int16 are ntohs as they are copied over.              *
 *                                                               *
 * uint32, int32 are ntohl as they are copied over.              *
 *                                                               *
 * signed conversions are wrapped around unsigned conversions    *
 * since marshalling should not interfere with signedness.       *
 *                                                               *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* make a copy of an unsigned byte from network. */
static uint8_t *uint8_copy_from_network(const uint8_t *data, size_t num)
{
    /* no marshalling needs to be done on individual bytes. */
    return uint8_copy_to_network(data, num);
}

/* make a copy of an signed byte from network. */
static int8_t *int8_copy_from_network(const uint8_t *data, size_t num)
{
    /* no marshalling needs to be done on individual bytes. */
    return int8_copy_to_network(data, num);
}

/* make a copy of an unsigned 16-bit integer from network. */
static uint16_t *uint16_copy_from_network(const void *data, size_t num)
{
    const uint16_t *uint16_data;
    uint16_t *uint16_copy;
    size_t i;

    uint16_copy = xmalloc((sizeof(uint16_t) * num));
    uint16_data = data;

    for(i = 0;i < num;i ++) {
        uint16_copy[i] = ntohs(uint16_data[i]);
    }

    return uint16_copy;
}

/* make a copy of an signed 16-bit integer. */
static int16_t *int16_copy_from_network(const void *data, size_t num)
{
    /* just cast. since it doesn't matter as long as we're just copying. */
    return (int16_t *)uint16_copy_from_network(data, num);
}

/* make a copy of an unsigned 32-bit integer. */
static uint32_t *uint32_copy_from_network(const void *data, size_t num)
{
    const uint32_t *uint32_data = data;
    uint32_t *uint32_copy;
    size_t i;

    uint32_copy = xmalloc((sizeof(uint32_t) * num));
    uint32_data = data;

    for(i = 0;i < num;i ++) {
        uint32_copy[i] = ntohl(uint32_data[i]);
    }

    return uint32_copy;
}

/* make a copy of an signed 16-bit integer. */
static int32_t *int32_copy_from_network(const void *data, size_t num)
{
    /* just cast. since it doesn't matter as long as we're just copying. */
    return (int32_t *)uint32_copy_to_network(data, num); 
}

/* convenience routine. take any size/typ eand convert to a list. */
static list_t *int_list_from_network_proc(const uint8_t *data, size_t len, size_t size, uint8_t sign)
{
    list_t *data_list;
    void *datum;

    /* check for at least one atom. */
    if(len < size)
        return NULL;

    /* check to make sure it's divisible by exactly the size of the atom. */
    if(len%size)
        return NULL;

    data_list = list_create();

    while(len)  {

        if(sign) {

            switch(size) {

            case sizeof(int8_t):
                datum = int8_copy_from_network(data, 1);
                break;

            case sizeof(int16_t):
                datum = int16_copy_from_network(data, 1);
                break;

            case sizeof(int32_t):
                datum = int32_copy_from_network(data, 1);
                break;

            default:
                FATAL_MESSAGE("illegal size passed for conversion. this is a bug report me.");
                exit(1);
            }

        } else {

            switch(size) {

            case sizeof(uint8_t):
                datum = uint8_copy_from_network(data, 1);
                break;

            case sizeof(uint16_t):
                datum = uint16_copy_from_network(data, 1);
                break;

            case sizeof(uint32_t):
                datum = uint32_copy_from_network(data, 1);
                break;

            default:
                FATAL_MESSAGE("illegal size passed for conversion. this is a bug report me.");
                exit(1);
            }
        }

        if(datum == NULL) {
            list_destroy(data_list, xfree);
            return NULL;
        }

        /* add datum to list. */
        list_add(data_list, datum);

        /* decrement length, and push pointer forward. */
        len -= size;
        data += size;
    }

    return data_list;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * From internal string to internal format.                      *
 *                                                               *
 * All input is passed via a string. The string is re-read from  *
 * a file we've already created (usually), or rewritten to be    *
 * handled internally. This means we can trust it.               *
 *                                                               *
 * scanfs throughout with the necessary C99 macro to read        *
 * in the value.                                                 *
 *                                                               *
 * The only special case happens when we want to read into a     *
 * list or an array. Here we expect a string with null           *
 * characters to terminate the individual datums. This is done   *
 * by marking the special delimiter ';'.                         *                 
 *                                                               *
 *                                                               *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* internal string to internal unsigned byte value. */
static uint8_t *internal_string_uint8_to_internal(const char *input)
{
    uint8_t *uint8_val;

    uint8_val = xmalloc(sizeof(uint8_t));

    sscanf(input, "%"SCNu8, uint8_val);

    return uint8_val;
}

/* internal string to internal unsigned byte value. */
static uint16_t *internal_string_uint16_to_internal(const char *input)
{
    uint16_t *uint16_data;

    uint16_data = xmalloc(sizeof(uint16_t));

    sscanf(input, "%"SCNu16, uint16_data);

    return uint16_data;
}

/* internal string to internal unsigned byte value. */
static uint32_t *internal_string_uint32_to_internal(const char *input)
{
    uint32_t *uint32_data;

    uint32_data = xmalloc(sizeof(uint32_t));

    sscanf(input, "%"SCNu32, uint32_data);

    return uint32_data;
}

/* internal string to internal signed 8-bit value. */
static int8_t *internal_string_int8_to_internal(const char *input)
{
    int8_t *int8_val;

    int8_val = xmalloc(sizeof(int8_t));

    sscanf(input, "%"SCNi8, int8_val);

    return int8_val;
}

/* internal string to internal byte value. */
static int16_t *internal_string_int16_to_internal(const char *input)
{
    int16_t *int16_val;

    int16_val = xmalloc(sizeof(int16_t));

    sscanf(input, "%"SCNi16, int16_val);

    return int16_val;
}

/* internal string to internal byte value. */
static int32_t *internal_string_int32_to_internal(const char *input)
{
    int32_t *int32_val;

    int32_val = xmalloc(sizeof(int32_t));

    sscanf(input, "%"SCNi32, int32_val);

    return int32_val;
}

/* * * * * * * * * * * * * * * * * * *
 * internal string to integer array. *
 * * * * * * * * * * * * * * * * * * */

static void *internal_string_to_array_int_proc(const char *input, size_t count, size_t size, uint8_t sign)
{
    void *data;
    size_t i;
    const char *ptr;

    uint8_t  *uint8_ptr;
    uint16_t *uint16_ptr;
    uint32_t *uint32_ptr;

    int8_t  *int8_ptr;
    int16_t *int16_ptr;
    int32_t *int32_ptr;

    data = xmalloc(count * size);

    ptr = input;
    i = 0;

    while(i < count)  {

        if(sign) {

            switch(size) {

            case sizeof(int8_t):
                int8_ptr = data;
                sscanf(ptr, "%"SCNi8, (int8_ptr + i));
                break;

            case sizeof(int16_t):
                int16_ptr = data;
                sscanf(ptr, "%"SCNi16, (int16_ptr + i));
                break;

            case sizeof(int32_t):
                int32_ptr = data;
                sscanf(ptr, "%"SCNi32, (int32_ptr + i));
                break;

            default:
                FATAL_MESSAGE("illegal size passed. this is a bug report me.");
                exit(1);
            }

        } else {

            switch(size) {

            case sizeof(uint8_t):
                uint8_ptr = data;
                sscanf(ptr, "%"SCNu8, (uint8_ptr + i));
                break;

            case sizeof(uint16_t):
                uint16_ptr = data;
                sscanf(ptr, "%"SCNu16, (uint16_ptr + i));
                break;

            case sizeof(uint32_t):
                uint32_ptr = data;
                sscanf(ptr, "%"SCNu32, (uint32_ptr + i));
                break;

            default:
                FATAL_MESSAGE("illegal size passed. this is a bug report me.");
                exit(1);
            }
        }

        for(;*ptr;ptr++); /* skip. */
        ptr++;
    }

    return data;
}

static list_t *internal_string_to_list_int_proc(const char *input, size_t count, size_t size, uint8_t sign)
{
    list_t *data_list;
    size_t i;
    const char *ptr;

    uint8_t  *uint8_data;
    uint16_t *uint16_data;
    uint32_t *uint32_data;

    int8_t  *int8_data;
    int16_t *int16_data;
    int32_t *int32_data;

    data_list = list_create();
    ptr = input;
    i = 0;

    while(i < count)  {

        if(sign) {

            switch(size) {

            case sizeof(int8_t):
                int8_data = xmalloc(sizeof(int8_t));
                sscanf(ptr, "%"SCNi8, int8_data);
                list_add(data_list, int8_data);

                break;

            case sizeof(int16_t):
                int16_data = xmalloc(sizeof(int16_t));
                sscanf(ptr, "%"SCNi16, int16_data);
                list_add(data_list, int16_data);

                break;

            case sizeof(int32_t):
                int32_data = xmalloc(sizeof(int32_t));
                sscanf(ptr, "%"SCNi32, int32_data);
                list_add(data_list, int32_data);

                break;

            default:
                FATAL_MESSAGE("illegal size passed. this is a bug report me.");
                exit(1);
            }

        } else {

            switch(size) {

            case sizeof(uint8_t):
                uint8_data = xmalloc(sizeof(uint8_t));
                sscanf(ptr, "%"SCNu8, uint8_data);
                list_add(data_list, uint8_data);

                break;

            case sizeof(uint16_t):
                uint16_data = xmalloc(sizeof(uint16_t));
                sscanf(ptr, "%"SCNu16, uint16_data);
                list_add(data_list, uint16_data);

                break;

            case sizeof(uint32_t):
                uint32_data = xmalloc(sizeof(uint32_t));
                sscanf(ptr, "%"SCNu32, uint32_data);
                list_add(data_list, uint32_data);

                break;

            default:
                FATAL_MESSAGE("illegal size passed. this is a bug report me.");
                exit(1);
            }
        }

        for(;*ptr;ptr++); /* skip. */
        ptr++;
    }

    return data_list;
}

static ip_addr_t *internal_string_ip_addr_to_internal(const char *input)
{
    ip_addr_t *ip_addr;
    in_addr_t in_addr;

    ip_addr = xmalloc(sizeof(ip_addr_t));
    in_addr = inet_addr(input);
    memcpy(ip_addr, &in_addr, IP_ADDR_LEN);

    return ip_addr;
}

static list_t *internal_string_list_ip_addr_to_internal(const char *input, size_t count)
{
    list_t *ip_addr_list;
    ip_addr_t *ip_addr;
    const char *ptr;
    size_t i;

    ip_addr_list = list_create();

    i = 0;
    ptr = input;

    while(i < count)  {

        ip_addr  = internal_string_ip_addr_to_internal(ptr);
        list_add(ip_addr_list, ip_addr);

        for(;*ptr;ptr++) /* skip past sentinel. */
            ptr++;

        i++;
    }

    return ip_addr_list;
}

/* procified to string routines. */

static char *uint8_to_string_proc(void *data, size_t num, const char *separator)
{
    stringbuffer_t *sb;
    char *str;
    uint8_t *uint8_ptr;
    size_t i;

    uint8_ptr = data;
    sb = stringbuffer_create();

    uint8_ptr = data;
    if(num == 1) {

        stringbuffer_aprintf(sb, "%"PRIu8, *uint8_ptr);

    } else {

        for(i = 0;i < num;i++) {
            stringbuffer_aprintf(sb, "%"PRIu8"%s", uint8_ptr[i], separator);
        }
    }

    str = xstrdup(stringbuffer_getstring(sb));
    stringbuffer_destroy(sb);

    return str;
}

static char *uint16_to_string_proc(void *data, size_t num, char *separator)
{
    stringbuffer_t *sb;
    char *str;
    uint16_t *uint16_ptr;
    size_t i;

    uint16_ptr = data;
    sb = stringbuffer_create();

    if(num == 1) {

        stringbuffer_aprintf(sb, "%"PRIu16, *uint16_ptr);

    } else {

        for(i = 0;i < num;i++) {
            stringbuffer_aprintf(sb, "%"PRIu16"%s", uint16_ptr[i], separator);
        }
    }

    str = xstrdup(stringbuffer_getstring(sb));
    stringbuffer_destroy(sb);

    return str;
}

static char *uint32_to_string_proc(void *data, size_t num, const char *separator)
{
    stringbuffer_t *sb;
    char *str;
    uint32_t *uint32_ptr;
    size_t i;

    uint32_ptr = data;
    sb = stringbuffer_create();

    if(num == 1) {

        stringbuffer_aprintf(sb, "%"PRIu32, *uint32_ptr);

    } else {

        for(i = 0;i < num;i++) {
            stringbuffer_aprintf(sb, "%"PRIu32"%s", uint32_ptr[i], separator);
        }
    }

    str = xstrdup(stringbuffer_getstring(sb));
    stringbuffer_destroy(sb);

    return str;
}

static char *int8_to_string_proc(void *data, size_t num, const char *separator) 
{
    stringbuffer_t *sb;
    char *str;
    int8_t *int8_ptr;
    size_t i;

    int8_ptr = data;
    sb = stringbuffer_create();

    if(num == 1) {

        stringbuffer_aprintf(sb, "%"PRIi8, *int8_ptr);

    } else {

        for(i = 0;i < num;i++) {
            stringbuffer_aprintf(sb, "%"PRIi8"%s", int8_ptr[i], separator);
        }
    }

    str = xstrdup(stringbuffer_getstring(sb));
    stringbuffer_destroy(sb);

    return str;
}

static char *int16_to_string_proc(void *data, size_t num, const char *separator)
{
    stringbuffer_t *sb;
    char *str;
    int16_t *int16_ptr;
    size_t i;

    int16_ptr = data;
    sb = stringbuffer_create();

    if(num == 1) {

        stringbuffer_aprintf(sb, "%"PRIi16, *int16_ptr);

    } else {

        for(i = 0;i < num;i++) {
            stringbuffer_aprintf(sb, "%"PRIi16"%s", int16_ptr[i], separator);
        }
    }

    str = xstrdup(stringbuffer_getstring(sb));
    stringbuffer_destroy(sb);

    return str;
}

static char *int32_to_string_proc(void *data, size_t num, const char *separator)
{
    stringbuffer_t *sb;
    char *str;
    int32_t *int32_ptr;
    size_t i;

    int32_ptr = data;
    sb = stringbuffer_create();

    if(num == 1) {

        stringbuffer_aprintf(sb, "%"PRIi32, *int32_ptr);

    } else {

        for(i = 0;i < num;i++) {
            stringbuffer_aprintf(sb, "%"PRIi32"%s", int32_ptr[i], separator);
        }
    }

    str = xstrdup(stringbuffer_getstring(sb));
    stringbuffer_destroy(sb);

    return str;
}

static char *int_list_to_string_proc(void *list_data, size_t size, uint8_t sign, const char *separator)
{
    int32_t  *int32_ptr;
    int16_t  *int16_ptr;
    int8_t   *int8_ptr;
    uint32_t *uint32_ptr;
    uint16_t *uint16_ptr;
    uint8_t  *uint8_ptr;
    void *data;
    size_t i;

    stringbuffer_t *sb;
    char *str;
    list_t *list;

    sb = stringbuffer_create();
    list = list_data;
    list_rewind(list);

    i = 0;
    while((data = list_next(list)) != NULL) {

        if(sign) {

            switch(size) {

            case sizeof(int8_t):

                i++;
                int8_ptr = data;

                if(i < list_get_len(list)) {
                    stringbuffer_aprintf(sb, "%"PRIi8"%s", *int8_ptr, separator);
                } else {
                    stringbuffer_aprintf(sb, "%"PRIi8, *int8_ptr);
                }
                break;

            case sizeof(int16_t):

                i++;
                int16_ptr = data;

                if(i < list_get_len(list)) {
                    stringbuffer_aprintf(sb, "%"PRIi16"%s", *int16_ptr, separator);
                } else {
                    stringbuffer_aprintf(sb, "%"PRIi16, *int16_ptr);
                }
                break;

            case sizeof(int32_t):

                i++;
                int32_ptr = data;

                if(i < list_get_len(list)) {
                    stringbuffer_aprintf(sb, "%"PRIi32"%s", *int32_ptr, separator);
                } else {
                    stringbuffer_aprintf(sb, "%"PRIi32, *int32_ptr);
                }
                break;
            }

        } else {

            switch (size) {

            case sizeof(uint8_t):

                i++;
                uint8_ptr = data;

                if(i < list_get_len(list)) {
                    stringbuffer_aprintf(sb, "%"PRIu8"%s", *uint8_ptr, separator);
                } else {
                    stringbuffer_aprintf(sb, "%"PRIu8, *uint8_ptr);
                }
                break;

            case sizeof(int16_t):

                i++;
                uint16_ptr = data;

                if(i < list_get_len(list)) {
                    stringbuffer_aprintf(sb, "%"PRIu16"%s", *uint16_ptr, separator);
                } else {
                    stringbuffer_aprintf(sb, "%"PRIu16, *uint16_ptr);
                }
                break;

            case sizeof(int32_t):

                i++;
                uint32_ptr = data;

                if(i < list_get_len(list)) {
                    stringbuffer_aprintf(sb, "%"PRIu32"%s", *uint32_ptr, separator);
                } else {
                    stringbuffer_aprintf(sb, "%"PRIu32, *uint32_ptr);
                }
                break;
            }
        }
    }

    str = xstrdup(stringbuffer_getstring(sb));
    stringbuffer_destroy(sb);

    return str;
}

static char *ip_addr_to_string_proc(void *data, uint8_t quoted)
{
    stringbuffer_t *sb;
    ip_addr_t *ip_addr;
    char *ip_addr_str;

    ip_addr = data;
    ip_addr_str = ip_addr_to_string(*ip_addr);

    if(quoted) {
        sb = stringbuffer_create();
        stringbuffer_aprintf(sb, "\"%s\"", ip_addr_str);

        xfree(ip_addr_str);
        ip_addr_str = xstrdup(stringbuffer_getstring(sb));
    }

    return ip_addr_str;
}

static char *ip_addr_list_to_string_proc(void *data, const char *separator, uint8_t quoted)
{
    ip_addr_t *ip_addr;
    list_t *list;
    stringbuffer_t *sb;
    size_t num;
    char *str, *ip_addr_str;

    sb = stringbuffer_create();

    list = data;
    list_rewind(list);
    num = 0;

    while((ip_addr = list_next(list)) != NULL) {

        ip_addr_str = ip_addr_to_string(*ip_addr);
        num++;
        if(num < list_get_len(list)) {
            stringbuffer_aprintf(sb, "%s%s", ip_addr_str, separator);
        } else {
            stringbuffer_aprintf(sb, "%s", ip_addr_str);
        }

        xfree(ip_addr_str);
    }

    str = xstrdup(stringbuffer_getstring(sb));
    stringbuffer_destroy(sb);

    return str;
}

/* from internal data to user string. */

static char *uint8_to_user_string(void *data, size_t num)
{
    return uint8_to_string_proc(data, num, ", ");
}

static char *uint16_to_user_string(void *data, size_t num)
{
    return uint16_to_string_proc(data, num, ", ");
}

static char *uint32_to_user_string(void *data, size_t num)
{
    return uint32_to_string_proc(data, num, ", ");
}

static char *int8_to_user_string(void *data, size_t num)
{
    return int8_to_string_proc(data, num, ", ");
}

static char *int16_to_user_string(void *data, size_t num)
{
    return int16_to_string_proc(data, num, ", ");
}

static char *int32_to_user_string(void *data, size_t num)
{
    return int32_to_string_proc(data, num, ", ");
}

static char *int_list_to_user_string(void *list_data, size_t size, uint8_t sign)
{
    return int_list_to_string_proc(list_data, size, sign, ", ");
}

static char *ip_addr_list_to_user_string(void *data)
{
    return ip_addr_list_to_string_proc(data, ", ", 1);
}

static char *ip_addr_to_user_string(void *data)
{
    return ip_addr_to_string_proc(data, 1);
}

/* from internal data to internal string */

static char *uint8_to_internal_string(void *data, size_t num)
{
    return uint8_to_string_proc(data, num, ";");
}

static char *uint16_to_internal_string(void *data, size_t num)
{
    return uint16_to_string_proc(data, num, ";");
}

static char *uint32_to_internal_string(void *data, size_t num)
{
    return uint32_to_string_proc(data, num, ";");
}

static char *int8_to_internal_string(void *data, size_t num)
{
    return int8_to_string_proc(data, num, ";");
}

static char *int16_to_internal_string(void *data, size_t num)
{
    return int16_to_string_proc(data, num, ";");
}

static char *int32_to_internal_string(void *data, size_t num)
{
    return int32_to_string_proc(data, num, ";");
}

static char *int_list_to_internal_string(void *list_data, size_t size, uint8_t sign)
{
    return int_list_to_string_proc(list_data, size, sign, ";");
}

static char *ip_addr_list_to_internal_string(void *data)
{
    return ip_addr_list_to_string_proc(data, ";", 0);
}

static char *ip_addr_to_internal_string(void *data)
{
    return ip_addr_to_string_proc(data, 0);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * *
 * public interface.                                 *
 *                                                   *
 * This is where all the above routines are called   *
 * to create a dhcp option, or extract data          *
 * from a dhcp option.                               * 
 * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* * * * * * * * * * * * * * * * * * *
 * from network data to dhcp option  *
 * * * * * * * * * * * * * * * * * * */

int dhcp_opt_from_network_uint8(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    if(len != sizeof(uint8_t))
        return 1;

    opt->val = uint8_copy_from_network(data, 1);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_network_uint16(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    if(len != sizeof(uint16_t))
        return 1;

    opt->val = uint16_copy_from_network(data, 1);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_network_uint32(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    if(len != sizeof(uint32_t))
        return 1;

    opt->val = uint32_copy_from_network(data, 1);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_network_int8(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    if(len != sizeof(int8_t))
        return 1;

    opt->val = int8_copy_from_network(data, 1);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_network_int16(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    if(len != sizeof(int16_t))
        return 1;

    opt->val = int16_copy_from_network(data, 1);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_network_int32(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    if(len != sizeof(int32_t))
        return 1;

    opt->val = int32_copy_from_network(data, 1);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_network_list_uint8(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    list_t *uint8_list;

    uint8_list = int_list_from_network_proc(data, len, sizeof(uint8_t), 0);

    if(uint8_list == NULL)
        return 1;

    opt->val = uint8_list;
    opt->num = len/sizeof(uint8_t);

    return 0;
}

int dhcp_opt_from_network_list_uint16(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
   list_t *uint16_list;

    uint16_list = int_list_from_network_proc(data, len, sizeof(uint16_t), 0);

    if(uint16_list == NULL)
        return 1;

    opt->val = uint16_list;
    opt->num = len/sizeof(uint16_t);

    return 0;
}

int dhcp_opt_from_network_list_uint32(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
   list_t *uint32_list;

   uint32_list = int_list_from_network_proc(data, len, sizeof(uint32_t), 0);

   if(uint32_list == NULL)
       return 1;

   opt->val = uint32_list;
   opt->num = len/sizeof(uint32_t);

   return 0;
}

int dhcp_opt_from_network_list_int8(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    list_t *int8_list;

    int8_list = int_list_from_network_proc(data, len, sizeof(int8_t), 1);

    if(int8_list == NULL)
        return 1;

    opt->val = int8_list;
    opt->num = len/sizeof(int8_t);

    return 0;        
}

int dhcp_opt_from_network_list_int16(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    list_t *int16_list;

    int16_list = int_list_from_network_proc(data, len, sizeof(int16_t), 1);

    if(int16_list == NULL)
        return 1;

    opt->val = int16_list;
    opt->num = len/sizeof(int16_t);

    return 0;        
}

int dhcp_opt_from_network_list_int32(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    list_t *int32_list;

    int32_list = int_list_from_network_proc(data, len, sizeof(int32_t), 1);

    if(int32_list == NULL)
        return 1;

    opt->val = int32_list;
    opt->num = len/sizeof(int32_t);

    return 0;        
}

int dhcp_opt_from_network_ip_addr(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    uint8_t *datum;

    datum = uint8_copy_from_network(data, IP_ADDR_LEN);

    opt->val = datum;
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_network_array_uint8(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    size_t num;

    num = (len/sizeof(uint8_t));
    opt->val = uint8_copy_from_network(data, num);
    opt->num = num;

    return 0;
}

int dhcp_opt_from_network_array_uint16(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    size_t num;

    if(len%sizeof(uint16_t))
        return 1;

    num = (len/sizeof(uint16_t));
    opt->val = uint16_copy_from_network(data, num);
    opt->num = num;

    return 0;
}

int dhcp_opt_from_network_array_uint32(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    size_t num;

    if(len%sizeof(uint32_t))
        return 1;

    num = (len/sizeof(uint32_t));
    opt->val = uint32_copy_from_network(data, num);
    opt->num = num;

    return 0;
}

int dhcp_opt_from_network_array_int8(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    size_t num;

    num = (len/sizeof(int8_t));
    opt->val = int8_copy_from_network(data, num);
    opt->num = num;

    return 0;
}

int dhcp_opt_from_network_array_int16(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    size_t num;

    if(len%sizeof(int16_t))
        return 1;

    num = (len/sizeof(int16_t));
    opt->val = int16_copy_from_network(data, num);
    opt->num = num;

    return 0;
}

int dhcp_opt_from_network_array_int32(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    size_t num;

    if(len%sizeof(int32_t))
        return 1;

    num = (len/sizeof(int32_t));
    opt->val = int32_copy_from_network(data, num);
    opt->num = num;

    return 0;
}

int dhcp_opt_from_network_ip_addr_pair(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    /* this is handled the same as converting a list of unsigned 32 bit integer. */
    return dhcp_opt_from_network_list_ip_addr(opt, data, len);
}

int dhcp_opt_from_network_list_ip_addr(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    list_t *address_list;
    void *datum;
    size_t num = 0;

    if(len%sizeof(ip_addr_t))
        return 1;

    address_list = list_create();

    while(len) {

        datum = uint8_copy_from_network(data, IP_ADDR_LEN);
        list_add(address_list, datum);

        data += sizeof(ip_addr_t);
        len -= sizeof(ip_addr_t);
        num++;
    }

    opt->val = address_list;
    opt->num = num;

    return 0;
}

int dhcp_opt_from_network_list_ip_addr_pair(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    /* this is handled the same as converting a list of addresses. */
    return dhcp_opt_from_network_list_ip_addr(opt, data, len);
}

int dhcp_opt_from_network_string(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    size_t string_len;
    uint8_t null_flag;
    int8_t *new_string;

    /* Make a copy. One issue we have is the use of the null
     * character half way through the string. Do we care? Yes we
     * do. Our internal string routines may use nulls as
     * sentinels. If this breaks things that's too bad. */

    /* Read up to a null or the end. If a null is present set a flag
     * so we don't null terminate. */

    null_flag = 0;
    for(string_len = 0; string_len < len; string_len++) {
        if(data[string_len] == 0) {
            null_flag = 1;
            break;
        }
    }

    if(null_flag) {

        new_string = xmalloc(sizeof(int8_t) * string_len);
        memcpy(new_string, data, string_len);
        opt->num = (string_len - 1);

    } else {

        new_string = xmalloc(sizeof(int8_t) * (string_len + 1));
        memcpy(new_string, data, string_len);
        new_string[string_len] = 0;
        opt->num = string_len;

    }

    opt->val = new_string;

    return 0;
}

int dhcp_opt_from_network_nvt_string(dhcp_opt_t *opt, const uint8_t *data, size_t len)
{
    /* make sure it is seven bit clean (NVT) */
    if(!is_seven_bit_clean(data, len))  {
        return 1;
    }

    /* then we can just call the copy from network string routine. */
    return dhcp_opt_from_network_string(opt, data, len);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * dhcp_opt from internal string conversion routines.  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */

int dhcp_opt_from_internal_string_uint8(dhcp_opt_t *opt, const char *input)
{
    opt->val = internal_string_uint8_to_internal(input);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_internal_string_uint16(dhcp_opt_t *opt, const char *input)
{
    opt->val = internal_string_uint16_to_internal(input);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_internal_string_uint32(dhcp_opt_t *opt, const char *input)
{
    opt->val = internal_string_uint32_to_internal(input);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_internal_string_int8(dhcp_opt_t *opt, const char *input)
{
    opt->val = internal_string_int8_to_internal(input);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_internal_string_int16(dhcp_opt_t *opt, const char *input)
{
    opt->val = internal_string_int16_to_internal(input);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_internal_string_int32(dhcp_opt_t *opt, const char *input)
{
    opt->val = internal_string_int32_to_internal(input);
    opt->num = 1;

    return 0;
}

static int dhcp_opt_from_internal_string_list_int_proc(dhcp_opt_t *opt, const char *input,
                                                        size_t size, uint8_t sign)
{
    char *string_copy;
    size_t count;

    string_copy = xstrdup(input);
    count = count_internal_string_atoms_and_mark(string_copy);

    opt->val = internal_string_to_list_int_proc(string_copy, count, size, sign);
    opt->num = count;

    xfree(string_copy);
    return 0;
}

static int dhcp_opt_from_internal_string_array_int_proc(dhcp_opt_t *opt, const char *input,
                                                        size_t size, uint8_t sign)
{
    char *string_copy;
    size_t count;

    string_copy = xstrdup(input);
    count = count_internal_string_atoms_and_mark(string_copy);

    opt->val = internal_string_to_array_int_proc(string_copy, count, size, sign);
    opt->num = count;

    xfree(string_copy);
    return 0;
}

int dhcp_opt_from_internal_string_array_uint8(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_array_int_proc(opt, input, sizeof(uint8_t), 0);
}

int dhcp_opt_from_internal_string_array_uint16(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_array_int_proc(opt, input, sizeof(uint16_t), 0);
}

int dhcp_opt_from_internal_string_array_uint32(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_array_int_proc(opt, input, sizeof(uint32_t), 0);
}

int dhcp_opt_from_internal_string_array_int8(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_array_int_proc(opt, input, sizeof(int8_t), 1);
}

int dhcp_opt_from_internal_string_array_int16(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_array_int_proc(opt, input, sizeof(int16_t), 1);
}

int dhcp_opt_from_internal_string_array_int32(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_array_int_proc(opt, input, sizeof(int32_t), 1);
}

int dhcp_opt_from_internal_string_list_uint8(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_list_int_proc(opt, input, sizeof(uint8_t), 0);
}

int dhcp_opt_from_internal_string_list_uint16(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_list_int_proc(opt, input, sizeof(uint16_t), 0);
}

int dhcp_opt_from_internal_string_list_uint32(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_list_int_proc(opt, input, sizeof(uint32_t), 0);
}

int dhcp_opt_from_internal_string_list_int8(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_list_int_proc(opt, input, sizeof(int8_t), 1);
}

int dhcp_opt_from_internal_string_list_int16(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_list_int_proc(opt, input, sizeof(int16_t), 1);
}

int dhcp_opt_from_internal_string_list_int32(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_list_int_proc(opt, input, sizeof(int32_t), 1);
}

int dhcp_opt_from_internal_string_ip_addr(dhcp_opt_t *opt, const char *input)
{
    opt->val = internal_string_ip_addr_to_internal(input);
    opt->num = 1;

    return 0;
}

int dhcp_opt_from_internal_string_ip_addr_pair(dhcp_opt_t *opt, const char *input)
{
    char *string_copy;
    size_t count;

    string_copy = xstrdup(input);
    count = count_internal_string_atoms_and_mark(string_copy);

    opt->val = internal_string_list_ip_addr_to_internal(string_copy, count);
    opt->num = count;

    xfree(string_copy);
    return 0;
}

int dhcp_opt_from_internal_string_list_ip_addr(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_ip_addr_pair(opt, input);
}

int dhcp_opt_from_internal_string_list_ip_addr_pair(dhcp_opt_t *opt, const char *input)
{
    return dhcp_opt_from_internal_string_ip_addr_pair(opt, input);
}

int dhcp_opt_from_internal_string_string(dhcp_opt_t *opt, const char *input)
{
    opt->val = xstrdup(input);
    opt->num = strlen(input);

    return 0;
}

int dhcp_opt_from_internal_string_nvt_string(dhcp_opt_t *opt, const char *input)
{
    if(!is_seven_bit_clean(input, strlen(input)))  {
        return 1;
    }

    return dhcp_opt_from_internal_string_string(opt, input);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * *
 * dhcp_opt from user string conversion routines:  *
 * * * * * * * * * * * * * * * * * * * * * * * * * */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * all these routines can be wrapped around the internal       *
 * routines. we just need to take them from list format to an  *
 * internal string format which is fairly easy.                *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static int dhcp_opt_from_user_string_proc(dhcp_opt_t *opt, list_t *input_list, int (*convert)(dhcp_opt_t *opt, const char *string))
{
    const char *input_string;

    input_string = list_first(input_list);

    if(input_string == NULL)  {
        return 1;
    }

    return convert(opt, input_string);
}

static int dhcp_opt_from_user_string_group_proc(dhcp_opt_t *opt, list_t *input_list, 
                                                int (*convert)(dhcp_opt_t *opt, const char *string))
{
    stringbuffer_t *sb;
    const char *input_string;
    int retval;

    sb = stringbuffer_create();

    list_rewind(input_list);
    while((input_string = list_next(input_list)) != NULL)  {

        stringbuffer_aprintf(sb, "%s;", input_string);
    }

    retval = convert(opt, stringbuffer_getstring(sb));

    stringbuffer_destroy(sb);
    return retval;
    
}

/* integer values. */
int dhcp_opt_from_user_string_uint8(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_proc(opt, input_list, dhcp_opt_from_internal_string_uint8);
}

int dhcp_opt_from_user_string_uint16(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_proc(opt, input_list, dhcp_opt_from_internal_string_uint16);
}

int dhcp_opt_from_user_string_uint32(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_proc(opt, input_list, dhcp_opt_from_internal_string_uint32);
}

int dhcp_opt_from_user_string_int8(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_proc(opt, input_list, dhcp_opt_from_internal_string_int8);
}

int dhcp_opt_from_user_string_int16(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_proc(opt, input_list, dhcp_opt_from_internal_string_int16);
}

int dhcp_opt_from_user_string_int32(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_proc(opt, input_list, dhcp_opt_from_internal_string_int32);
}

int dhcp_opt_from_user_string_list_uint8(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_list_uint8);
}

int dhcp_opt_from_user_string_list_uint16(dhcp_opt_t *opt,list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_list_uint16);
}

int dhcp_opt_from_user_string_list_uint32(dhcp_opt_t *opt,list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_list_uint32);
}

int dhcp_opt_from_user_string_list_int8(dhcp_opt_t *opt,list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_list_int8);
}

int dhcp_opt_from_user_string_list_int16(dhcp_opt_t *opt,list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_list_int16);
}

int dhcp_opt_from_user_string_list_int32(dhcp_opt_t *opt,list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_list_int32);
}

/* arrays of integer values. */
int dhcp_opt_from_user_string_array_uint8(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_array_uint8);
}

int dhcp_opt_from_user_string_array_uint16(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_array_uint16);
}

int dhcp_opt_from_user_string_array_uint32(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_array_uint32);
}

int dhcp_opt_from_user_string_array_int8(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_array_int8);
}

int dhcp_opt_from_user_string_array_int16(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_array_int16);
}

int dhcp_opt_from_user_string_array_int32(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_array_int32);
}

/* ip address and ip address pairs. */
int dhcp_opt_from_user_string_ip_addr(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_proc(opt, input_list, dhcp_opt_from_internal_string_ip_addr);
}

int dhcp_opt_from_user_string_ip_addr_pair(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_ip_addr_pair);
}

/* list of addresses and ip address pairs. */
int dhcp_opt_from_user_string_list_ip_addr(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_list_ip_addr);
}

int dhcp_opt_from_user_string_list_ip_addr_pair(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_group_proc(opt, input_list, dhcp_opt_from_internal_string_list_ip_addr_pair);
}

/* strings. */
int dhcp_opt_from_user_string_nvt_string(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_proc(opt, input_list, dhcp_opt_from_internal_string_nvt_string);
}

int dhcp_opt_from_user_string_string(dhcp_opt_t *opt, list_t *input_list)
{
    return dhcp_opt_from_user_string_proc(opt, input_list, dhcp_opt_from_internal_string_string);
}

/* * * * * * * * * * * * * * * *
 * dhcp_opt to network data.   *
 * * * * * * * * * * * * * * * */

void *dhcp_opt_to_network_uint8(dhcp_opt_t *opt)
{
    return uint8_copy_to_network(opt->val, 1);
}

void *dhcp_opt_to_network_uint16(dhcp_opt_t *opt)
{
    return uint16_copy_to_network(opt->val, 1);
}

void *dhcp_opt_to_network_uint32(dhcp_opt_t *opt)
{
    return uint32_copy_to_network(opt->val, 1);
}

void *dhcp_opt_to_network_int8(dhcp_opt_t *opt)
{
    return int8_copy_to_network(opt->val, 1);
}

void *dhcp_opt_to_network_int16(dhcp_opt_t *opt)
{
    return int16_copy_to_network(opt->val, 1);
}

void *dhcp_opt_to_network_int32(dhcp_opt_t *opt)
{
    return int32_copy_to_network(opt->val, 1);
}

void *dhcp_opt_to_network_list_uint8(dhcp_opt_t *opt)
{
    return int_list_copy_to_network(opt->val, sizeof(uint8_t), 0);
}

void *dhcp_opt_to_network_list_uint16(dhcp_opt_t *opt)
{
    return int_list_copy_to_network(opt->val, sizeof(uint16_t), 0);
}

void *dhcp_opt_to_network_list_uint32(dhcp_opt_t *opt)
{
    return int_list_copy_to_network(opt->val, sizeof(uint32_t), 0);
}

void *dhcp_opt_to_network_list_int8(dhcp_opt_t *opt)
{
    return int_list_copy_to_network(opt->val, sizeof(int8_t), 1);
}

void *dhcp_opt_to_network_list_int16(dhcp_opt_t *opt)
{
    return int_list_copy_to_network(opt->val, sizeof(int16_t), 1);
}

void *dhcp_opt_to_network_list_int32(dhcp_opt_t *opt)
{
    return int_list_copy_to_network(opt->val, sizeof(int32_t), 1);
}

void *dhcp_opt_to_network_array_uint8(dhcp_opt_t *opt)
{
    return uint8_copy_to_network(opt->val, opt->num);
}

void *dhcp_opt_to_network_array_uint16(dhcp_opt_t *opt)
{
    return uint16_copy_to_network(opt->val, opt->num);
}

void *dhcp_opt_to_network_array_uint32(dhcp_opt_t *opt)
{
    return uint32_copy_to_network(opt->val, opt->num);
}

void *dhcp_opt_to_network_array_int8(dhcp_opt_t *opt)
{
    return int8_copy_to_network(opt->val, opt->num);
}

void *dhcp_opt_to_network_array_int16(dhcp_opt_t *opt)
{
    return int16_copy_to_network(opt->val, opt->num);
}

void *dhcp_opt_to_network_array_int32(dhcp_opt_t *opt)
{
    return int32_copy_to_network(opt->val, opt->num);
}

void *dhcp_opt_to_network_ip_addr(dhcp_opt_t *opt)
{
    return uint8_copy_to_network(opt->val, IP_ADDR_LEN);
}

void *dhcp_opt_to_network_ip_addr_pair(dhcp_opt_t *opt)
{
    return dhcp_opt_to_network_list_ip_addr(opt);
}

void *dhcp_opt_to_network_list_ip_addr_pair(dhcp_opt_t *opt)
{
    return dhcp_opt_to_network_list_ip_addr(opt);
}

void *dhcp_opt_to_network_list_ip_addr(dhcp_opt_t *opt)
{
    list_t *list;
    ip_addr_t *ip_address;
    ip_addr_t *data;
    size_t i;

    /* take a list of ip addresses and create an array of them.
     * that's all we're really doing. */

    list = opt->val;
    list_rewind(list);

    data = xmalloc(IP_ADDR_LEN * list_get_len(list));

    i = 0;
    while((ip_address = list_next(list)) != NULL)  {
        memcpy(&data[i], ip_address, IP_ADDR_LEN);
        i++;
    }

    return data;
}

void *dhcp_opt_to_network_nvt_string(dhcp_opt_t *opt)
{
    /* do not incorporate a terminating NULL.  we may need to
     * create seperate to network converters for any options
     * which require a terminating null. */

    /* FIXME: find out if any options require a terminating NULL. */

    int8_t *network_str, *str;

    str = opt->val;
    network_str = xmalloc(sizeof(int8_t) * opt->num);
    memcpy(network_str, str, opt->num);

    return network_str;
}

void *dhcp_opt_to_network_string(dhcp_opt_t *opt)
{
    return dhcp_opt_to_network_nvt_string(opt);
}

/* * * * * * * * * * * * * * * *
 * dhcp_opt to user string.    *
 * * * * * * * * * * * * * * * */

char *dhcp_opt_to_user_string_uint8(dhcp_opt_t *opt)
{
    return uint8_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_uint16(dhcp_opt_t *opt)
{
    return uint16_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_uint32(dhcp_opt_t *opt)
{
    return uint32_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_int8(dhcp_opt_t *opt)
{
    return int8_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_int16(dhcp_opt_t *opt)
{
    return int16_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_int32(dhcp_opt_t *opt)
{
    return int32_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_list_uint8(dhcp_opt_t *opt)
{
    return int_list_to_user_string(opt->val, sizeof(uint8_t), 0);
}

char *dhcp_opt_to_user_string_list_uint16(dhcp_opt_t *opt)
{
    return int_list_to_user_string(opt->val, sizeof(uint16_t), 0);
}

char *dhcp_opt_to_user_string_list_uint32(dhcp_opt_t *opt)
{
    return int_list_to_user_string(opt->val, sizeof(uint32_t), 0);
}

char *dhcp_opt_to_user_string_list_int8(dhcp_opt_t *opt)
{
    return int_list_to_user_string(opt->val, sizeof(int8_t), 1);
}

char *dhcp_opt_to_user_string_list_int16(dhcp_opt_t *opt)
{
    return int_list_to_user_string(opt->val, sizeof(int16_t), 1);
}

char *dhcp_opt_to_user_string_list_int32(dhcp_opt_t *opt)
{
    return int_list_to_user_string(opt->val, sizeof(int32_t), 1);
}

char *dhcp_opt_to_user_string_array_uint8(dhcp_opt_t *opt)
{
    return uint8_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_array_uint16(dhcp_opt_t *opt)
{
    return uint16_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_array_uint32(dhcp_opt_t *opt)
{
    return uint32_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_array_int8(dhcp_opt_t *opt)
{
    return int8_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_array_int16(dhcp_opt_t *opt)
{    
    return int16_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_array_int32(dhcp_opt_t *opt)
{
    return int32_to_user_string(opt->val, opt->num);
}

char *dhcp_opt_to_user_string_ip_addr(dhcp_opt_t *opt)
{
    return ip_addr_to_user_string(opt->val);
}

char *dhcp_opt_to_user_string_ip_addr_pair(dhcp_opt_t *opt)
{
    return ip_addr_list_to_user_string(opt->val);
}

char *dhcp_opt_to_user_string_list_ip_addr(dhcp_opt_t *opt)
{
    return ip_addr_list_to_user_string(opt->val);
}

char *dhcp_opt_to_user_string_list_ip_addr_pair(dhcp_opt_t *opt)
{
    return ip_addr_list_to_user_string(opt->val);
}

char *dhcp_opt_to_user_string_nvt_string(dhcp_opt_t *opt)
{
    return xstrdup((char *)opt->val);
}

char *dhcp_opt_to_user_string_string(dhcp_opt_t *opt)
{
    return xstrdup((char *)opt->val);
}

/* * * * * * * * * * * * * * * * * *
 * dhcp_opt to internal string.    *
 * * * * * * * * * * * * * * * * * */

char *dhcp_opt_to_internal_string_uint8(dhcp_opt_t *opt)
{
    return uint8_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_uint16(dhcp_opt_t *opt)
{
    return uint16_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_uint32(dhcp_opt_t *opt)
{
    return uint32_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_int8(dhcp_opt_t *opt)
{
    return int8_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_int16(dhcp_opt_t *opt)
{
    return int16_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_int32(dhcp_opt_t *opt)
{
    return int32_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_list_uint8(dhcp_opt_t *opt)
{
    return int_list_to_internal_string(opt->val, sizeof(uint8_t), 0);
}

char *dhcp_opt_to_internal_string_list_uint16(dhcp_opt_t *opt)
{
    return int_list_to_internal_string(opt->val, sizeof(uint16_t), 0);
}

char *dhcp_opt_to_internal_string_list_uint32(dhcp_opt_t *opt)
{
    return int_list_to_internal_string(opt->val, sizeof(uint32_t), 0);
}

char *dhcp_opt_to_internal_string_list_int8(dhcp_opt_t *opt)
{
    return int_list_to_internal_string(opt->val, sizeof(int8_t), 1);
}

char *dhcp_opt_to_internal_string_list_int16(dhcp_opt_t *opt)
{
    return int_list_to_internal_string(opt->val, sizeof(int16_t), 1);
}

char *dhcp_opt_to_internal_string_list_int32(dhcp_opt_t *opt)
{
    return int_list_to_internal_string(opt->val, sizeof(int32_t), 1);
}

char *dhcp_opt_to_internal_string_array_uint8(dhcp_opt_t *opt)
{
    return uint8_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_array_uint16(dhcp_opt_t *opt)
{
    return uint16_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_array_uint32(dhcp_opt_t *opt)
{
    return uint32_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_array_int8(dhcp_opt_t *opt)
{
    return int8_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_array_int16(dhcp_opt_t *opt)
{    
    return int16_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_array_int32(dhcp_opt_t *opt)
{
    return int32_to_internal_string(opt->val, opt->num);
}

char *dhcp_opt_to_internal_string_ip_addr(dhcp_opt_t *opt)
{
    return ip_addr_to_internal_string(opt->val);
}

char *dhcp_opt_to_internal_string_ip_addr_pair(dhcp_opt_t *opt)
{
    return ip_addr_list_to_internal_string(opt->val);
}

char *dhcp_opt_to_internal_string_list_ip_addr(dhcp_opt_t *opt)
{
    return ip_addr_list_to_internal_string(opt->val);
}

char *dhcp_opt_to_internal_string_list_ip_addr_pair(dhcp_opt_t *opt)
{
    return ip_addr_list_to_internal_string(opt->val);
}

char *dhcp_opt_to_internal_string_nvt_string(dhcp_opt_t *opt)
{
    return xstrdup((char *)opt->val);
}

char *dhcp_opt_to_internal_string_string(dhcp_opt_t *opt)
{
    return xstrdup((char *)opt->val);
}
