#ifndef DHCP_OPTION_CONVERT_H
#define DHCP_OPTION_CONVERT_H

/* dhcp option conversion routines. */

/* from network data host data. */

extern int dhcp_opt_from_network_uint8(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_uint16(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_uint32(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_int8(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_int16(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_int32(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_list_uint8(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_list_uint16(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_list_uint32(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_list_int8(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_list_int16(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_list_int32(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_array_uint8(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_array_uint16(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_array_uint32(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_array_int8(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_array_int16(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_array_int32(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_ip_addr(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_ip_addr_pair(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_list_ip_addr(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_list_ip_addr_pair(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_nvt_string(dhcp_opt_t *opt, const uint8_t *data, size_t len);
extern int dhcp_opt_from_network_string(dhcp_opt_t *opt, const uint8_t *data, size_t len);

/* from internal string to host data. */

extern int dhcp_opt_from_internal_string_uint8(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_uint16(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_uint32(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_int8(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_int16(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_int32(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_list_uint8(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_list_uint16(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_list_uint32(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_list_int8(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_list_int16(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_list_int32(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_array_uint8(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_array_uint16(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_array_uint32(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_array_int8(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_array_int16(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_array_int32(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_ip_addr(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_ip_addr_pair(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_list_ip_addr(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_list_ip_addr_pair(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_nvt_string(dhcp_opt_t *opt, const char *input);
extern int dhcp_opt_from_internal_string_string(dhcp_opt_t *opt, const char *input);

/* from user string to host data */

extern int dhcp_opt_from_user_string_uint8(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_uint16(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_uint32(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_int8(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_int16(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_int32(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_list_uint8(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_list_uint16(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_list_uint32(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_list_int8(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_list_int16(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_list_int32(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_array_uint8(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_array_uint16(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_array_uint32(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_array_int8(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_array_int16(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_array_int32(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_ip_addr(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_ip_addr_pair(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_list_ip_addr(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_list_ip_addr_pair(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_nvt_string(dhcp_opt_t *opt, list_t *input_list);
extern int dhcp_opt_from_user_string_string(dhcp_opt_t *opt, list_t *input_list);

/* to network data. */

extern void *dhcp_opt_to_network_uint8(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_uint16(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_uint32(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_int8(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_int16(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_int32(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_list_uint8(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_list_uint16(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_list_uint32(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_list_int8(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_list_int16(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_list_int32(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_array_uint8(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_array_uint16(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_array_uint32(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_array_int8(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_array_int16(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_array_int32(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_ip_addr(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_ip_addr_pair(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_list_ip_addr(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_list_ip_addr_pair(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_nvt_string(dhcp_opt_t *opt);
extern void *dhcp_opt_to_network_string(dhcp_opt_t *opt);

/* to user string. */

extern char *dhcp_opt_to_user_string_uint8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_uint16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_uint32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_int8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_int16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_int32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_list_uint8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_list_uint16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_list_uint32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_list_int8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_list_int16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_list_int32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_array_uint8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_array_uint16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_array_uint32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_array_int8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_array_int16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_array_int32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_ip_addr(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_ip_addr_pair(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_list_ip_addr(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_list_ip_addr_pair(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_nvt_string(dhcp_opt_t *opt);
extern char *dhcp_opt_to_user_string_string(dhcp_opt_t *opt);

/* to internal string. */

extern char *dhcp_opt_to_internal_string_uint8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_uint16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_uint32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_int8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_int16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_int32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_list_uint8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_list_uint16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_list_uint32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_list_int8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_list_int16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_list_int32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_array_uint8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_array_uint16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_array_uint32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_array_int8(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_array_int16(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_array_int32(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_ip_addr(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_ip_addr_pair(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_list_ip_addr(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_list_ip_addr_pair(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_nvt_string(dhcp_opt_t *opt);
extern char *dhcp_opt_to_internal_string_string(dhcp_opt_t *opt);

#endif /* DHCP_OPTION_CONVERT_H */
