/* $Header: /cvsroot/dhcp-agent/dhcp-agent/src/dhcp-print.c,v 1.8 2003/06/28 02:08:24 actmodern Exp $
 *
 * Copyright 2002 Thamer Alharbash
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Print functions.
 *
 */

#define MODULE_NAME "dhcp-print"

#include "dhcp-local.h"
#include "dhcp-limits.h"
#include "dhcp-libutil.h"
#include "dhcp-librawnet.h"
#include "dhcp-option.h"

#include "dhcp-print.h"

/* Print the fixed header briefly.*/
void print_dhcphdr_brief(dhcp_obj * dhcp)
{
    uint8_t op;
    struct in_addr in;
    uint8_t *hw_ptr;
    int i;

    /* op */
    op = dhcp_get_op(dhcp);
    printf("op:");

    switch (op) {

    case DHCP_BOOTP_REQUEST:
        printf("req");
        break;

    case DHCP_BOOTP_REPLY:
        printf("rpy");
        break;

    default:
        printf("%-3d", op);
    }

    printf(" ");

    /* Hardware Type field. */

    printf("hd:");
    switch (dhcp_get_htype(dhcp)) {

    case DLT_NULL:
        printf("%s", "nul");
        break;

    case DLT_EN10MB:
    case DLT_EN3MB:
        printf("%s", "eth");
        break;

    case DLT_AX25:
        printf("%s", "ax25");
        break;

    case DLT_PRONET:
        printf("%s", "pnet");
        break;

    case DLT_CHAOS:
        printf("%s", "chas");
        break;

    case DLT_ARCNET:
        printf("%s", "slip");
        break;

    case DLT_PPP:
        printf("%s", "ppp");
        break;

    case DLT_FDDI:
        printf("%s", "fddi");
        break;

    default:
        printf("%s", "unkn");
        break;

    }
    printf(" ");

    printf("hlen:%d ", dhcp_get_hlen(dhcp));
    printf("hops:%d ", dhcp_get_hops(dhcp));
    printf("xid:%x ", dhcp_get_xid(dhcp));
    printf("secs:%d ", dhcp_get_secs(dhcp));

    if(dhcp_get_flags(dhcp))
        printf("f:brdcst");
    else
        printf("f:none");
    printf(" ");

    in.s_addr = dhcp_get_ciaddr(dhcp);
    printf("ciaddr: %s ", inet_ntoa(in));

    in.s_addr = dhcp_get_yiaddr(dhcp);
    printf("yiaddr: %s ", inet_ntoa(in));

    in.s_addr = dhcp_get_siaddr(dhcp);
    printf("siaddr: %s ", inet_ntoa(in));

    in.s_addr = dhcp_get_giaddr(dhcp);
    printf("agaddr: %s ", inet_ntoa(in));

    printf("chdr: ");
    hw_ptr = dhcp_get_chaddr(dhcp);
    for(i = 0; i < 16; i++) {
        printf("%.2x", *hw_ptr);
        hw_ptr++;
    }
    printf(" ");
    fflush(stdout);
}


/* Print the fixed header verbosely.
 * 
 * We know that if a dhcp packet is read successfully into a dhcp
 * object we'll always have the fixed header in its completeness,
 * although it may be malformed. So we can just print out here
 * as best we can without any worries about length.
 * 
 */

void print_dhcphdr_verbose(dhcp_obj * dhcp)
{
    struct in_addr in;
    uint8_t *hw_ptr;
    int i;
    uint8_t op;

    printf("----------------------------------DHCP Info--------------------------\n");

    /* Op field. */

    printf("[Operation: ");
    op = dhcp_get_op(dhcp);

    switch (op) {

    case DHCP_BOOTP_REQUEST:
        printf("%-12s] ", "BootRequest");
        break;

    case DHCP_BOOTP_REPLY:
        printf("%-12s] ", "BootReply");
        break;

    default:
        printf("%-12d] ", op);
    }

    /* Hardware Type field. */

    printf("[Hardware Type: ");
    switch (dhcp_get_htype(dhcp)) {

    case DLT_NULL:
        printf("%-8s ]", "Null");
        break;

    case DLT_EN10MB:
    case DLT_EN3MB:
        printf("%-8s ] ", "Ethernet");
        break;

    case DLT_AX25:
        printf("%-8s ] ", "AX25");
        break;

    case DLT_PRONET:
        printf("%-8s ] ", "ProNet");
        break;

    case DLT_CHAOS:
        printf("%-8s ] ", "Chaos");
        break;

    case DLT_ARCNET:
        printf("%-8s ] ", "SLIP");
        break;

    case DLT_PPP:
        printf("%-8s ] ", "PPP");
        break;

    case DLT_FDDI:
        printf("%-8s ] ", "FDDI");
        break;

    default:
        printf("%-7s ] ", "Unknown");
        break;

    }

    printf("\n");
    printf("[Hardware Addr Len: %-3d ] ", dhcp_get_hlen(dhcp));
    printf("[Hops: %-17d ] ", dhcp_get_hops(dhcp));
    printf("[XID: %-8X ]", dhcp_get_xid(dhcp));
    printf("\n");

    printf("[Seconds: %-14d] ", dhcp_get_secs(dhcp));

    if(dhcp_get_flags(dhcp))
        printf("[Flags: Broadcast]");
    else
        printf("[Flags: ]");
    printf("\n\n");

    printf("    ");
    in.s_addr = dhcp_get_ciaddr(dhcp);
    printf("[Client Addr: %-15s]  ", inet_ntoa(in));

    in.s_addr = dhcp_get_yiaddr(dhcp);
    printf("[Your Addr: %-16s]", inet_ntoa(in));
    printf("\n");

    printf("    ");
    in.s_addr = dhcp_get_siaddr(dhcp);
    printf("[Server Addr: %-15s]  ", inet_ntoa(in));

    in.s_addr = dhcp_get_giaddr(dhcp);
    printf("[Agent Addr: %-15s]", inet_ntoa(in));
    printf("\n\n");
    printf("         ");

    printf("[Client HW Addr: ");
    hw_ptr = dhcp_get_chaddr(dhcp);
    for(i = 0; i < 16; i++) {
        printf("%.2x", *hw_ptr);
        hw_ptr++;
    }
    printf("]\n");

    /* Ignoring name and file field for now. Too long.
     * I'll need to find a way to display it properly. */

    printf("---------------------------------------------------------------------\n");

    fflush(stdout);
    return;
}

void print_request_options(const uint8_t *data, int len, int printval)
{
    int i;

    printf("\n---------------------------------------------------------------------\n");
    printval = 0;

    for(i = 0; i < len; i++) {

        if(data[i] > MAX_OPTIONS_HANDLED)
            continue;

        printval += printf("%s", option_handlers[data[i]].prefix);

        if(i == (len - 1))
            printf(".");
        else
            printf(", ");

        if(printval >= 50) {
            printf("\n");
            printval = 0;
        }
    }
    printf("\n---------------------------------------------------------------------\n");

    return;
}

void print_dhcp_type_message(const uint8_t *data, int len, int printval)
{
    uint8_t dhcp_type = *data;

    if(len != 1) {
        printf("<Malformed Data>");
        return;
    }

    switch (dhcp_type) {

    case DHCP_DISCOVER_TM:
        printf("DISCOVER");
        break;

    case DHCP_OFFER_TM:
        printf("OFFER");
        break;

    case DHCP_REQUEST_TM:
        printf("REQUEST");
        break;

    case DHCP_DECLINE_TM:
        printf("DECLINE");
        break;

    case DHCP_DHCPNAK_TM:
        printf("DHCPNAK");
        break;

    case DHCP_DHCPACK_TM:
        printf("DHCPACK");
        break;

    case DHCP_RELEASE_TM:
        printf("RELEASE");
        break;

    default:
        break;
    }

    return;
}

/* Just print a 32bit inet address. */

static int print_address(const uint8_t *data)
{
    struct in_addr in;

    memcpy(&in.s_addr, data, 4);
    return (printf("%s", inet_ntoa(in)));
}

/* Print a list of addresses. Works on single
 * addresses too. */

void print_address_list(const uint8_t *data, int len, int ini_val)
{
    int i, j, line_val;

    if(len <= 0 || len % 4 != 0) {
        printf("<Malformed data>");
        return;
    }

    line_val = ini_val;

    for(i = 0; i < len; i += 4) {

        line_val += print_address(data);
        data += 4;

        if(i >= (len - 4))
            printf(".");
        else
            printf(", ");

        if(line_val > 60) {
            printf("\n");
            line_val = ini_val;

            for(j = 0; j < line_val; j++)
                printf(" ");
        }
    }

    return;
}

/* Same as print_address_list only we add a netmask in between. */

void print_address_mask_list(const uint8_t *data, int len, int ini_val)
{
    int i, j, line_val;

    if(len <= 0 || len % 8 != 0) {
        printf("<Malformed data>");
        return;
    }

    line_val = ini_val;

    for(i = 0; i < len; i += 4) {

        line_val += print_address(data);
        data += 4;

        if(i != 0 && (i % 8 == 0)) {
            /* We're at a mask. */
            printf("/");
            continue;
        }

        if(i >= (len - 4))
            printf(".");
        else
            printf(", ");

        if(line_val > 60) {
            printf("\n");
            line_val = ini_val;

            for(j = 0; j < line_val; j++)
                printf(" ");
        }
    }

    return;
}

/* Same as print_address_list only we add a route with "->" in between. */

void print_address_route_list(const uint8_t *data, int len, int ini_val)
{
    int i, j, line_val;

    if(len <= 0 || len % 8 != 0) {
        printf("<Malformed data>");
        return;
    }

    line_val = ini_val;

    for(i = 0; i < len; i += 4) {

        line_val += print_address(data);
        data += 4;

        if(i != 0 && (i % 8 == 0)) {
            /* We're at a mask. */
            printf("->");
            continue;
        }

        if(i >= (len - 4))
            printf(".");
        else
            printf(", ");

        if(line_val > 60) {
            printf("\n");
            line_val = ini_val;

            for(j = 0; j < line_val; j++)
                printf(" ");
        }
    }

    return;
}

/* Print a string. */

void print_string(const uint8_t *data, int len, int print_val)
{
    char *new_string;

    if(len > MAX_STRING_LEN) {
        printf("<String too long to display>");
        return;
    }

    new_string = xmalloc(sizeof(char) * (len + 1));
    strncpy(new_string, data, len);
    new_string[len] = 0;

    printf("%s", new_string);

    xfree(new_string);

    return;
}

/* Print a signed 32bit integer. */

void print_int32(const uint8_t *data, int len, int print_val)
{
    bpf_int32 val;

    if(len != 4) {
        printf("<Malformed data>");
        return;
    }

    memcpy(&val, data, 4);
    val = ntohl(val);
    printf("%d", val);
    return;
}

/* Print unsigned 32-bit integer. */

void print_uint32(const uint8_t *data, int len, int print_val)
{
    uint32_t val;

    if(len != 4) {
        printf("<Malformed data>");
        return;
    }


    memcpy(&val, data, 4);
    val = ntohl(val);
    printf("%u", val);

    return;
}

/* Print unsigned 16-bit integer. */

void print_uint16(const uint8_t *data, int len, int print_val)
{
    uint16_t val;

    if(len != 2) {
        printf("<Malformed data>");
        return;
    }

    memcpy(&val, data, 2);
    val = ntohs(val);
    printf("%u", val);

    return;
}

/* Print unsigned byte. */

void print_ubyte(const uint8_t *data, int len, int print_val)
{
    if(len != 1) {
        printf("<Malformed data>");
        return;
    }

    printf("%u", (*data));
    return;
}

void print_uint16_list(const uint8_t *data, int len, int ini_val)
{
    int i, j, line_val;
    uint16_t val;

    if(len <= 0 || len % 2 != 0) {
        printf("<Malformed data>");
        return;
    }

    line_val = ini_val;

    for(i = 0; i < len; i += 2) {

        val = *(data);
        line_val += printf("%u", val);
        data += 2;


        if(i < (len - 2))
            printf(", ");

        if(line_val > 60) {
            printf("\n");
            line_val = ini_val;

            for(j = 0; j < line_val; j++)
                printf(" ");
        }
    }

    return;
}

/* Print a boolean value as enabled or disabled. */

void print_en_bool(const uint8_t *data, int len, int print_val)
{

    if(len != 1) {
        printf("<Malformed data>");
        return;
    }

    if(*data == 0)
        printf("Disabled");
    else
        printf("Enabled");

    return;
}

/* Print boolean: "Yes" "No" */

void print_yn_bool(const uint8_t *data, int len, int print_val)
{

    if(len != 1) {
        printf("<Malformed data>");
        return;
    }

    if(*data == 0)
        printf("Yes");
    else
        printf("No");

    return;
}

/* Print netbios node type. */

void print_netbios_node_type(const uint8_t *data, int len, int print_val)
{
    if(len != 1) {
        printf("<Malformed data>");
        return;
    }

    if((*data) && NETBIOS_B_NODE)
        printf("B-Node ");
    if((*data) && NETBIOS_P_NODE)
        printf("P-Node ");
    if((*data) && NETBIOS_M_NODE)
        printf("M-Node ");
    if((*data) && NETBIOS_H_NODE)
        printf("H-Node ");

    return;
}

/* Don't handle handler -- the irony... */

void dont_handle(const uint8_t *data, int len, int print_val)
{
    printf("<Not handled>");
    return;
}

/* Print options.
 *
 * We have an array with all the handlers.
 * We just check if the len, and tag field are legal
 * and then pass the data onto the print structure after
 * printing the prefix.
 *
 */

void print_options_verbose(dhcp_obj * dhcp)
{
    dhcp_opt_t *option;
    uint8_t tag;
    int print_val;
    uint8_t *val;

    dhcp_reset_option_seek(dhcp);

    while((option = dhcp_get_next_option(dhcp)) != NULL) {

        /* Since it's already been copied into the dhcp object
         * we can assume that the code will always be valid. */

        /* FIXME: eventually this will have to be rewritten.  the
         * sniffer, prior to the new dhcp object, expected data
         * in network form. just ask for it here. */

        tag = dhcp_opt_get_tag(option);
        print_val = printf("%s", option_handlers[tag].prefix);
        print_val += printf(": ");

        val = dhcp_opt_get_network_data(option);
        option_handlers[tag].handle_option(val,
                                           dhcp_opt_get_total_len(option), 
                                           print_val);
        xfree(val);
        printf("\n");
    }

    return;

}

/* Print source and destination address (brief). */
void print_ip_brief(ip_obj * ip)
{
    struct in_addr in;

    in.s_addr = ip_get_src_addr(ip);
    printf("srcip: %s ", inet_ntoa(in));

    in.s_addr = ip_get_dst_addr(ip);
    printf("dstip: %s ", inet_ntoa(in));
    return;
}

/* Print source and destination address (verbose). */
void print_ip_verbose(ip_obj * ip)
{
    struct in_addr in;

    printf("-------------------------------IP HDR--------------------------------\n");

    in.s_addr = ip_get_src_addr(ip);
    printf("Source: %-17s", inet_ntoa(in));
    printf("               ");

    in.s_addr = ip_get_dst_addr(ip);
    printf("Destination: %-17s\n", inet_ntoa(in));

    printf("---------------------------------------------------------------------\n");
    return;
}

void print_magic_cookie_brief(dhcp_obj * dhcp)
{
    struct in_addr in;

    memcpy(&in.s_addr, &dhcp->magic_cookie, 4);
    printf("cookie: %s ", inet_ntoa(in));
    return;
}

void print_magic_cookie_verbose(const char *status, dhcp_obj * dhcp)
{
    struct in_addr in;

    memcpy(&in.s_addr, &dhcp->magic_cookie, 4);

    printf("Cookie: %-6s : %-17s\n", status, inet_ntoa(in));
    printf("---------------------------------------------------------------------\n");

    return;
}

void print_timestamp_brief(struct timeval tm)
{
    printf("[%lu.%06lu] ", tm.tv_sec, tm.tv_usec);
    return;
}

void print_timestamp_verbose(struct timeval tm)
{
    printf("%lu.%06lu\n", tm.tv_sec, tm.tv_usec);
    return;
}

void print_eth_brief(eth_obj * eth)
{
    eth_addr_t src_eth_addr;
    eth_addr_t dst_eth_addr;

    src_eth_addr = eth_get_src_addr(eth);
    dst_eth_addr = eth_get_dst_addr(eth);

    printf("smac: %x:%x:%x:%x:%x:%x ", src_eth_addr.data[0], src_eth_addr.data[1],
           src_eth_addr.data[2], src_eth_addr.data[3], src_eth_addr.data[4], src_eth_addr.data[5]);
    printf("dmac: %x:%x:%x:%x:%x:%x ", dst_eth_addr.data[0], dst_eth_addr.data[1],
           dst_eth_addr.data[2], dst_eth_addr.data[3], dst_eth_addr.data[4], dst_eth_addr.data[5]);

    return;
}

void print_eth_verbose(eth_obj * eth)
{
    eth_addr_t src_eth_addr, dst_eth_addr;
    src_eth_addr = eth_get_src_addr(eth);
    dst_eth_addr = eth_get_dst_addr(eth);

    printf("--------------------------Ethernet Header-----------------------------\n");
    printf("SRC MAC: %x:%x:%x:%x:%x:%x", src_eth_addr.data[0], src_eth_addr.data[1],
           src_eth_addr.data[2], src_eth_addr.data[3], src_eth_addr.data[4], src_eth_addr.data[5]);
    printf("             DST MAC: %x:%x:%x:%x:%x:%x\n", dst_eth_addr.data[0], dst_eth_addr.data[1],
           dst_eth_addr.data[2], dst_eth_addr.data[3], dst_eth_addr.data[4], dst_eth_addr.data[5]);

    return;
}

void print_dhcp_packet_verbose(rawnet_t *net)
{
    print_eth_verbose(net->ether_p);
    print_ip_verbose(net->ip_p);
    print_dhcphdr_verbose(net->dhcp_p);
    if(dhcp_valid_magic_cookie(net->dhcp_p))
        print_magic_cookie_verbose("Good", net->dhcp_p);
    else
        print_magic_cookie_verbose("Bad", net->dhcp_p);
    print_options_verbose(net->dhcp_p);

    return;
}

void print_dhcp_packet_brief(rawnet_t *net)
{
    print_timestamp_brief(net->tstamp);
    print_eth_brief(net->ether_p);
    print_ip_brief(net->ip_p);
    print_dhcphdr_brief(net->dhcp_p);
    print_magic_cookie_brief(net->dhcp_p);

    return;
}
