/* $Header: /cvsroot/dhcp-agent/dhcp-agent/src/dhcp-tokenizer.h,v 1.1 2002/12/22 08:31:30 actmodern Exp $
 *
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com> 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * Tokenizer
 *
 */

#ifndef DHCP_TOKENIZER_H
#define DHCP_TOKENIZER_H

/* types. */

/* internal tokenizer type. */
typedef token_t (*tokenizer)(tokenizer_t *tokenizer);

/* structures. */

/* tokenizer dispatch structure. */
typedef struct {
    char character;
    tokenizer do_parse;
} tokenizers_t;

/* prototypes. */
extern const char *tokenizer_get_data(tokenizer_t *tokenizer);
extern int tokenizer_get_line_no(tokenizer_t *tokenizer);

extern tokenizer_t *tokenizer_create(const char *filename);
extern void tokenizer_destroy(tokenizer_t *tokenizer);

extern token_t tokenizer_peek_next_token_ignore_newlines(tokenizer_t *tokenizer);
extern token_t tokenizer_peek_next_token(tokenizer_t *tokenizer);

extern token_t tokenizer_get_next_token_ignore_newlines(tokenizer_t *tokenizer);
extern token_t tokenizer_get_next_token(tokenizer_t *tokenizer);

/* tokenizer return values. */
enum parser_atom { TOKEN_ERROR = 0, TOKEN_STRING, TOKEN_ASSIGNMENT, TOKEN_NEWLINE,
                   TOKEN_EOF, TOKEN_BLOCK_OPEN, TOKEN_BLOCK_CLOSE, TOKEN_SEMICOLON, 
                   TOKEN_COMMA };

#endif                          /* DHCP_TOKENIZER_H */
